/* Copyright (C) 1999 Aaron Lehmann
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include <gdk_imlib.h>

#include "gtkplot.h"
#include "gtkplotlayout.h"

#include "app.h"
#include "version.h"
#include "callbacks.h"
#include "dialogs.h"
#include "mathparsesym.h"
#include "mathparse.h"
#include "prefs.h"


extern GtkWidget *active_plot;
extern GtkWidget *window1, *ctlwin;
extern GtkWidget *layout;
extern GtkWidget *canvas;
extern GtkWidget *opmenu;


extern gint8      formn;
extern char      *formc [10];
extern GtkWidget *edit, *entry2, *entry3, *entry4, *entry6, *entry7, *entry8, *entry9;
extern GtkWidget *errlab;

extern int yyparse ();




void callback_popup (GtkWidget *w, gint8 *data)
{
  if (data)
    {
      gint8 formnum =  *data;
      
      strcpy (formc[formn], gtk_entry_get_text (GTK_ENTRY(edit)));
      formn = formnum;

      gtk_entry_set_text (GTK_ENTRY(edit), formc[formn]);
    }
}


void callback_button (GtkWidget *w, gpointer data)
{
  extern GtkWidget *edit;
  extern short eqnflag[10];
  extern char *c;
  extern char *errtext;
  extern short initial;
  extern short currset;
  extern short flagend;
  extern short parseerr;
  extern GtkPlotData *dataset[10];

  int i;
  char *h;

  strcpy (formc[formn], gtk_entry_get_text (GTK_ENTRY(edit))); /* A kind of sync */
  gtk_label_set_text (GTK_LABEL(errlab), "");

  //FIXME
  //prefs_read ();


  for (i = 0; i != 10; ++i)
    {
      if (*formc[i] != '\0')
	{
	  currset = i;
	  
	  mp_stack_clean ();
	  initial = 1;
	  flagend=0;
	  c = formc[i];
	  strcpy (errtext, "");
	  yyparse ();

	  if (!parseerr)
	    {
	      eqnflag[i] = 1;
	    }
	  else
	    {
	      eqnflag[i] = 0;
	    }
	}
      else { eqnflag[i] = 0; }
      parseerr = 0;
      if (eqnflag[i]) gtk_plot_show_dataset (dataset[i]);
      else gtk_plot_hide_dataset (dataset[i]);
    }
  h = (char *) malloc (500);
  gtk_label_get (GTK_LABEL(errlab), &h);
  if (!strcmp (h, ""))
    gtk_label_set_text (GTK_LABEL(errlab), "Ready.");
  gtk_widget_queue_draw (active_plot);
}


void callback_prefs (GtkWidget *w, gpointer data)
{
  dialog_prefs();
}


void callback_about (GtkWidget *w, gpointer data)
{
  dialog_about();
}


void callback_export (GtkWidget *w, gpointer data)
{
  GtkWidget *filew = gtk_file_selection_new (_("Export graph (Bitmap)"));
  gtk_signal_connect_after (GTK_OBJECT (GTK_FILE_SELECTION (filew)-> ok_button), "clicked", (GtkSignalFunc) callback_export_ok, filew);
  gtk_signal_connect_after (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), "graph.png");
  gtk_widget_show (filew);
}


void callback_export_ok (GtkWidget *w, GtkFileSelection *fs)
{
  /* Imlib ROCKS! Well it may be a memory hog and as leaky as a collandar, but it could do this in 3 lines. It took about 20 supercryptic lines of lowlevel pixmap code to make an export in graphtool, which only supports pixmap. */

  GdkImlibImage *im = gdk_imlib_create_image_from_drawable (GTK_PLOT_LAYOUT(layout)->pixmap, NULL, 0, 0, GTK_PLOT_LAYOUT(layout)->width, GTK_PLOT_LAYOUT(layout)->height);
  gdk_imlib_save_image (im, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), NULL);
  gtk_widget_destroy (GTK_WIDGET(fs));
  gdk_imlib_kill_image (im);
}


void callback_export_ps (GtkWidget *w, gpointer data)
{
  GtkWidget *filew = gtk_file_selection_new (_("Export graph (PostScript)"));
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)-> ok_button), "clicked", (GtkSignalFunc) callback_export_ps_ok, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), "graph.ps");
  gtk_widget_show (filew);
}


void callback_export_ps_ok (GtkWidget *w, GtkFileSelection *fs)
{
  gtk_plot_layout_export_ps_with_size(GTK_PLOT_LAYOUT(canvas), gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), 0, 0, GTK_PLOT_PSPOINTS, 612, 612);
  gtk_widget_destroy (GTK_WIDGET(fs));
}


void callback_print (GtkWidget *w, gpointer data)
{
  char s[65], cmd[69];
  tmpnam (s); /* Get a temporary file name */

  gtk_plot_layout_export_ps_with_size(GTK_PLOT_LAYOUT(canvas), s, 0, 0, GTK_PLOT_PSPOINTS, 612, 612);

  strcpy (cmd, "lpr ");
  strcat (cmd, s);

  /* Print it */
  system (cmd);

  unlink (s);
}


void callback_load (GtkWidget *w, gpointer data)
{
  GtkWidget *filew = gtk_file_selection_new (_("Load function set"));
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)-> ok_button), "clicked", (GtkSignalFunc) callback_load_ok, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show (filew);
}


void callback_save (GtkWidget *w, gpointer data)
{
  GtkWidget *filew = gtk_file_selection_new (_("Save function set"));
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)-> ok_button), "clicked", (GtkSignalFunc) callback_save_ok, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show (filew);
}


void callback_load_ok (GtkWidget *w, GtkFileSelection *fs)
{
  int i, len;
  FILE *file = fopen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "r");
  if (file)
    {
      for (i=0; i != 10; ++i)
	{
	  fgets (formc[i], 255, file);
	  len = strlen (formc[i]);
	  if (formc[i][len-1] == '\n') formc[i][len-1] = '\0'; /* gcc complains here if we use NULL */
	}
      fclose (file);
    }

  gtk_widget_destroy (GTK_WIDGET(fs));

  gtk_entry_set_text (GTK_ENTRY(edit), formc[formn]); /* A kind of sync */
}

void callback_save_ok (GtkWidget *w, GtkFileSelection *fs)
{
  FILE *file = fopen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "w");

  formc[formn] = gtk_entry_get_text (GTK_ENTRY(edit)); /* A kind of sync */

  if (file)
    {
      fprintf (file, "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
               formc[0],
	       formc[1],
	       formc[2],
	       formc[3],
	       formc[4],
	       formc[5],
	       formc[6],
	       formc[7],
	       formc[8],
	       formc[9]);
      
      fclose (file);
    }

  gtk_widget_destroy (GTK_WIDGET(fs));
}


void callback_source_drag_data_get (GtkWidget *widget,
				    GdkDragContext *context,
				    GtkSelectionData *selection_data,
				    guint info,
				    guint time,
				    guint data)
{
  char string[] = "Some String!";
  gtk_selection_data_set (selection_data, selection_data->target, 8, string, sizeof(string));
}


gboolean callback_keypress (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  gint8 formnum;

  if (event->string[0] == '\r')
    {
      callback_button (NULL, NULL);
    }

  else if(event->keyval == GDK_Down
	  || event->keyval == GDK_KP_Down
	  || event->keyval == GDK_ISO_Move_Line_Down
	  || event->keyval == GDK_Pointer_Down)
    {
      if (formn < 9)
	{
	  formnum = formn + 1;
	  gtk_option_menu_set_history (GTK_OPTION_MENU(opmenu), formnum);
	  callback_popup (NULL, &formnum);
	}
    }

  else if(event->keyval == GDK_Up
            || event->keyval == GDK_KP_Up
            || event->keyval == GDK_ISO_Move_Line_Up
            || event->keyval == GDK_Pointer_Up)
    {
      if (formn > 0)
	{
	  formnum = formn - 1;
	  gtk_option_menu_set_history (GTK_OPTION_MENU(opmenu), formnum);
	  callback_popup (NULL, &formnum);
	}
    }
  else return (TRUE);
  
  /* I have to do this to stop gtk from propagating this event;
     error in gtk? */
  event->keyval = GDK_Right;
  event->state = 0;
  event->length = 0;
  
  
  return (FALSE);
}


void callback_zoom_in (GtkWidget *w, gpointer data)
{
  extern double rangexmi, rangeymi, rangexma, rangeyma, majorticks_h, majorticks_v, minorticks_h, minorticks_v;
  char *nptr, ascii[32];
  double yeah = 2;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry4));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
    }

  rangexmi /= yeah;
  rangexma /= yeah;
  rangeymi /= yeah;
  rangeyma /= yeah;
  majorticks_h /= yeah;
  minorticks_h /= yeah;
  majorticks_v /= yeah;
  minorticks_v /= yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, majorticks_v, minorticks_v);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, majorticks_v, minorticks_v);

  sprintf (ascii, "%g", rangexmi);
  gtk_entry_set_text (GTK_ENTRY(entry6), ascii);
  sprintf (ascii, "%g", rangexma);
  gtk_entry_set_text (GTK_ENTRY(entry7), ascii);
  sprintf (ascii, "%g", rangeymi);
  gtk_entry_set_text (GTK_ENTRY(entry8), ascii);
  sprintf (ascii, "%g", rangeyma);
  gtk_entry_set_text (GTK_ENTRY(entry9), ascii);

  gtk_widget_queue_draw (layout);
}


void callback_zoom_out (GtkWidget *w, gpointer data)
{
  extern double rangexmi, rangeymi, rangexma, rangeyma, majorticks_h, majorticks_v, minorticks_h, minorticks_v;
  char *nptr, ascii[32];
  double yeah = 2;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry4));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
    }

  rangexmi *= yeah;
  rangexma *= yeah;
  rangeymi *= yeah;
  rangeyma *= yeah;
  majorticks_h *= yeah;
  minorticks_h *= yeah;
  majorticks_v *= yeah;
  minorticks_v *= yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, majorticks_v, minorticks_v);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, majorticks_v, minorticks_v);

  sprintf (ascii, "%g", rangexmi);
  gtk_entry_set_text (GTK_ENTRY(entry6), ascii);
  sprintf (ascii, "%g", rangexma);
  gtk_entry_set_text (GTK_ENTRY(entry7), ascii);
  sprintf (ascii, "%g", rangeymi);
  gtk_entry_set_text (GTK_ENTRY(entry8), ascii);
  sprintf (ascii, "%g", rangeyma);
  gtk_entry_set_text (GTK_ENTRY(entry9), ascii);

  gtk_widget_queue_draw (layout);
}


void callback_scroll_l (GtkWidget *w, gpointer data)
{
  extern void gtk_plot_layout_draw               (GtkWidget *widget, GdkRectangle *area);
  extern double rangexmi, rangeymi, rangexma, rangeyma;
  char *nptr;
  double yeah = 0.5;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry2));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
    }

  
  rangexmi -= yeah;
  rangexma -= yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_widget_queue_draw (layout);

  usleep (500);
}


void callback_scroll_r (GtkWidget *w, gpointer data)
{
  extern void gtk_plot_layout_draw               (GtkWidget *widget, GdkRectangle *area);
  extern double rangexmi, rangeymi, rangexma, rangeyma;
  char *nptr;
  double yeah = 0.5;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry2));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
    }

  
  rangexmi += yeah;
  rangexma += yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  usleep (500);
  gtk_widget_queue_draw (layout);
}


void callback_scroll_u (GtkWidget *w, gpointer data)
{
  extern void gtk_plot_layout_draw               (GtkWidget *widget, GdkRectangle *area);
  extern double rangexmi, rangeymi, rangexma, rangeyma;
  char *nptr;
  double yeah = 0.5;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry3));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
    }


  rangeymi += yeah;
  rangeyma += yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_widget_queue_draw (layout);

  usleep (500);
}


void callback_scroll_d (GtkWidget *w, gpointer data)
{
  extern void gtk_plot_layout_draw               (GtkWidget *widget, GdkRectangle *area);
  extern double rangexmi, rangeymi, rangexma, rangeyma;
  char *nptr;
  double yeah = 0.5;

  nptr = gtk_entry_get_text (GTK_ENTRY(entry3));
  yeah = g_strtod(nptr, (char **)NULL);

  if (yeah == 0)
    {
      return;
    }

  
  rangeymi -= yeah;
  rangeyma -= yeah;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_widget_queue_draw (layout);

  usleep (500);
}


void callback_range_apply (GtkWidget *w, gpointer data)
{
  extern void gtk_plot_layout_draw               (GtkWidget *widget, GdkRectangle *area);
  extern double rangexmi, rangeymi, rangexma, rangeyma, majorticks_h, majorticks_v, minorticks_h, minorticks_v;
  char *nptr;
  double yeah[4];

  nptr = gtk_entry_get_text (GTK_ENTRY(entry6));
  yeah[0] = g_strtod(nptr, (char **)NULL);

  nptr = gtk_entry_get_text (GTK_ENTRY(entry7));
  yeah[1] = g_strtod(nptr, (char **)NULL);

  nptr = gtk_entry_get_text (GTK_ENTRY(entry8));
  yeah[2] = g_strtod(nptr, (char **)NULL);

  nptr = gtk_entry_get_text (GTK_ENTRY(entry9));
  yeah[3] = g_strtod(nptr, (char **)NULL);

  /*  if (yeah[0] == 0 || yeah[1] == 0 || yeah[2] == 0 || yeah[3] == 0)
    {
      //ERROR Please enter a positive or negative numeric value
      return;
      }*/

  
  rangexmi = yeah[0];
  rangexma = yeah[1];
  rangeymi = yeah[2];
  rangeyma = yeah[3];

  majorticks_v = (rangexma - rangexmi) / 10;
  minorticks_v = (rangexma - rangexmi) / 20;
  majorticks_h = (rangeyma - rangeymi) / 10;
  minorticks_h = (rangeyma - rangeymi) / 20;

  gtk_plot_set_range (GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, majorticks_v, minorticks_v);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, majorticks_v, minorticks_v);
  gtk_widget_queue_draw (layout);
}


void callback_zoom_slider (GtkAdjustment *w, gpointer data)
{
  // put right new values in fields
  callback_range_apply (NULL, NULL);
}


void callback_ctl_show (GtkWidget *w, gpointer data)
{
 if (!GTK_IS_WIDGET(ctlwin))
 {
  return;
 }
 gtk_widget_show (ctlwin);
}


void callback_ctl_hide (GtkWidget *w, gpointer data)
{
  gtk_widget_hide (ctlwin);
}


void callback_label_x (GtkWidget *w, gpointer data)
{
  dialog_label (DIALOG_LABEL_X, " X Axis:   ", GTK_PLOT(active_plot)->bottom.title.text);
}


void callback_label_y (GtkWidget *w, gpointer data)
{
  dialog_label (DIALOG_LABEL_Y, " Y Axis:   ", GTK_PLOT(active_plot)->left.title.text);
}

