/* Copyright (C) 1999 Aaron Lehmann
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>

#include "gtkplot.h"

#include "app.h"
#include "version.h"
#include "prefs.h"
#include "dialogs.h"



struct cbk_record
{
  int type;
  GtkWidget *dlg, *text;
};

struct cbk_color
{
  int num;
  GtkWidget *colorsel;
};




extern GtkWidget *active_plot;
extern GtkWidget *layout;

extern struct prefs prf;
extern GtkPlotData *dataset[10];

extern void reset_flags (void);



void dialog_prefs (void)
{
  GtkWidget *prefdlg, *button, *chkbutton, *frame, *vbox1, *hbox1, *label;
  int i;
  char labname[20];


  prefdlg = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER(GTK_DIALOG(prefdlg)->vbox), 5);

  /* Create a Frame */
  frame = gtk_frame_new(NULL);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(prefdlg)->vbox), frame, FALSE, FALSE, 0);
  /* Set the frames label */
  gtk_frame_set_label( GTK_FRAME(frame), "Toggles" );

  /* Align the label at the right of the frame */
  gtk_frame_set_label_align( GTK_FRAME(frame), 1.0, 0.0);

  /* Set the style of the frame */
  gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show(frame);


  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER(frame), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER(vbox1), 5);
  gtk_widget_show (vbox1);

  chkbutton = gtk_check_button_new_with_label (_("Show legend"));
  gtk_box_pack_start (GTK_BOX(vbox1), chkbutton, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(chkbutton), prf.showlegend);
  gtk_widget_show (chkbutton);
  

    /* Create a Frame */
  frame = gtk_frame_new(NULL);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(prefdlg)->vbox), frame, FALSE, FALSE, 0);
  /* Set the frames label */
  gtk_frame_set_label( GTK_FRAME(frame), "Colors" );

  /* Align the label at the right of the frame */
  gtk_frame_set_label_align( GTK_FRAME(frame), 1.0, 0.0);

  /* Set the style of the frame */
  gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show(frame);


  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER(frame), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER(vbox1), 5);
  gtk_widget_show (vbox1);


  for (i=0; i != 10; ++i)
    {
      int *eye;
      /*double clrs[3];

      clrs[0] = prf.color[i].red/65535.0;
      clrs[1] = prf.color[i].green/65535.0;
      clrs[2] = prf.color[i].blue/65535.0;*/

      eye = (int *) malloc (sizeof (int));
      *eye = i;

      hbox1 = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
      gtk_widget_show (hbox1);
      sprintf (labname, "Function %i:", i+1);
      label = gtk_label_new (labname);
      gtk_box_pack_start (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
      gtk_widget_show (label);
      button = gtk_button_new ();
      gtk_box_pack_end (GTK_BOX(hbox1), button, FALSE, FALSE, 0);
      gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_prefs_color), eye);
      gtk_widget_show (button);
    }

  button = gtk_button_new_with_label (_("OK"));
  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(prefdlg)->action_area), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect_after (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_cancel), (void *) prefdlg);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_prefs_ok), chkbutton);
  button = gtk_button_new_with_label (_("Cancel"));
  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(prefdlg)->action_area), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_cancel), (void *) prefdlg);

  gtk_widget_show (prefdlg);
}


void dialog_prefs_ok (GtkWidget *w, gpointer data)
{
  prf.showlegend = GTK_TOGGLE_BUTTON(data)->active;
  reset_flags ();
  prefs_save ();
}


void dialog_prefs_color (GtkWidget *w, gpointer data)
{
  GtkWidget *colorsel;
  struct cbk_color *p;
  int *data2 =  (int *) data;
  p = (struct cbk_color *) malloc (sizeof (struct cbk_color));
  p -> num = *data2;
  free (data);

  colorsel = gtk_color_selection_dialog_new ("Color Selection");

  p -> colorsel = GTK_WIDGET(GTK_COLOR_SELECTION_DIALOG(colorsel)->colorsel);

  /*gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorsel)->colorsel), (gdouble *) data);*/
  gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG(colorsel)->help_button);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG(colorsel)->ok_button), "clicked", GTK_SIGNAL_FUNC (dialog_prefs_color_ok), p);
  gtk_signal_connect_after (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG(colorsel)->ok_button), "clicked", GTK_SIGNAL_FUNC (dialog_cancel), (void *) colorsel);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG(colorsel)->cancel_button), "clicked", GTK_SIGNAL_FUNC (dialog_cancel), colorsel);
  gtk_widget_show (colorsel);
}


void dialog_prefs_color_ok (GtkWidget *w, gpointer data)
{
  gdouble color[3];
  struct cbk_color *p = data;

  gtk_color_selection_get_color (GTK_COLOR_SELECTION(p->colorsel), &color);
  prf.color[p->num].red = (guint16)(color[0]*65535.0);
  prf.color[p->num].green = (guint16)(color[1]*65535.0);
  prf.color[p->num].blue = (guint16)(color[2]*65535.0);
  gdk_colormap_free_colors (gdk_colormap_get_system(), &prf.color[p->num], 1);
  gdk_color_alloc (gdk_colormap_get_system(), &prf.color[p->num]);
  gtk_plot_dataset_set_line_attributes(dataset[p->num], GTK_PLOT_LINE_SOLID, 0, prf.color[p->num]);
  free (p);
}


void dialog_cancel (GtkWidget *w, gpointer data)
{
  gtk_widget_destroy (GTK_WIDGET(data));
}


void dialog_about (void)
{
}


void dialog_label (int type, char *name, char *curr)
{
  GtkWidget *dialog, *button, *hbox1, *new, *label;
  struct cbk_record *rcd;
  rcd = (struct cbk_record *) malloc (sizeof(struct cbk_record));
  
  rcd->type = type;
  

  dialog = gtk_dialog_new ();

  rcd->dlg = dialog;

  hbox1=gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox1, FALSE, FALSE, 0);
  gtk_widget_show (hbox1);

  new = gtk_entry_new_with_max_length(50);
  gtk_entry_set_text (GTK_ENTRY(new), curr);
  gtk_box_pack_end (GTK_BOX(hbox1), new, FALSE, FALSE, 0);
  gtk_widget_show (new);
  rcd->text = new;

  label = gtk_label_new (name);
  gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  button = gtk_button_new_with_label (_("OK"));
  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_label_ok), rcd);
  button = gtk_button_new_with_label (_("Cancel"));
  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (dialog_cancel), dialog);

  gtk_widget_show (dialog);
}


void dialog_label_ok (GtkWidget *w, gpointer data)
{
  struct cbk_record *datas = (struct cbk_record *) data;
  switch (datas->type)
    {
    case DIALOG_LABEL_X:
      {
	double width;
	int w, height;
	
	gtk_plot_axis_set_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, gtk_entry_get_text (GTK_ENTRY(datas->text)));
	gtk_plot_text_get_size (GTK_PLOT(active_plot)->bottom.title, &w, &height);
	width = w;
	gtk_plot_axis_move_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, 0, 0.5 - (width/1000), GTK_PLOT(active_plot)->bottom.title.y);
	break;
      }
    case DIALOG_LABEL_Y:
      {
	double width;
	int height, w;
	
	gtk_plot_axis_set_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, gtk_entry_get_text (GTK_ENTRY(datas->text)));
	gtk_plot_text_get_size (GTK_PLOT(active_plot)->left.title, &height, &w);
	width = w;
	gtk_plot_axis_move_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, 0, GTK_PLOT(active_plot)->left.title.x, 0.5 - (width/1000));
	break;
      }
    }

  gtk_widget_queue_draw (layout);

  gtk_widget_destroy (datas->dlg);
  free (datas);
}
