/* Copyright (C) 1999 Aaron Lehmann
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk_imlib.h>
#include "gtkplot.h"
#include "gtkplotlayout.h"
#include "gtkplotcanvas.h"

#include "app.h"
#include "version.h"
#include "prefs.h"
#include "main.h"
#include "callbacks.h"
#include "calc.h"
#include "mathparsesym.h"
#include "mathparse.h"

GdkPixmap *pixmap=NULL;
GtkWidget **plots;
GtkWidget *active_plot;
GtkPlotData *dataset[10];
GtkWidget *window1, *ctlwin, *opmenu;
GtkWidget *layout;
GtkWidget *canvas;
GtkWidget *errlab;
GtkWidget *edit, *entry2, *entry3, *entry4, *entry6, *entry7, *entry8, *entry9;
gint nlayers = 0;
gint8      formn = 0;
char     *formc [10];
double rangexmi=-1, rangeymi=-1, rangexma=1, rangeyma=1, majorticks_h=0.2, majorticks_v=0.2, minorticks_h=0.1, minorticks_v=0.1;
short eqnflag[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };


extern struct prefs prf;

gdouble function0 (gdouble x, gboolean *err);
gdouble function1 (gdouble x, gboolean *err);
gdouble function2 (gdouble x, gboolean *err);
gdouble function3 (gdouble x, gboolean *err);
gdouble function4 (gdouble x, gboolean *err);
gdouble function5 (gdouble x, gboolean *err);
gdouble function6 (gdouble x, gboolean *err);
gdouble function7 (gdouble x, gboolean *err);
gdouble function8 (gdouble x, gboolean *err);
gdouble function9 (gdouble x, gboolean *err);
gdouble function(gdouble x, gboolean *err, short num);

void set_error (char *);


extern void get_main_menu (GtkWidget *window, GtkWidget **menubar);


gdouble function0 (gdouble x, gboolean *err)
{
 return function (x, err, 0);
}

gdouble function1 (gdouble x, gboolean *err)
{
  return function (x, err, 1);
}

gdouble function2 (gdouble x, gboolean *err)
{
  return function (x, err, 2);
}

gdouble function3 (gdouble x, gboolean *err)
{
  return function (x, err, 3);
}

gdouble function4 (gdouble x, gboolean *err)
{
  return function (x, err, 4);
}

gdouble function5 (gdouble x, gboolean *err)
{
  return function (x, err, 5);
}

gdouble function6 (gdouble x, gboolean *err)
{
  return function (x, err, 6);
}

gdouble function7 (gdouble x, gboolean *err)
{
  return function (x, err, 7);
}

gdouble function8 (gdouble x, gboolean *err)
{
  return function (x, err, 8);
}

gdouble function9 (gdouble x, gboolean *err)
{
  return function (x, err, 9);
}

gdouble function(gdouble x, gboolean *err, short num)
{
  gdouble result;
  extern short currset;

  currset=num;

  if (eqnflag[currset])
    {
      result = mp_eval_exp (x, 0);
      mp_fix_stack ();
      *err = FALSE;
    }
 else
   {
     *err = TRUE;
     result = 0;
   }
  return (result);
}


void *functions[] = { \
			    function0, \
			    function1, \
			    function2, \
			    function3, \
			    function4, \
			    function5, \
			    function6, \
			    function7, \
			    function8, \
			    function9};


GtkWidget *
new_layer(GtkWidget *canvas)
{
 nlayers++;

 plots = (GtkWidget **)g_realloc(plots, nlayers * sizeof(GtkWidget *));
 plots[nlayers-1] = gtk_plot_new_with_size(NULL, .75, .75);

 return plots[nlayers-1];
}


void
build_dataset(short num)
{
  char name [20];

 dataset[num] = gtk_plot_add_function(GTK_PLOT(active_plot), (GtkPlotFunc)functions[num]);
 gtk_plot_dataset_set_line_attributes(dataset[num],
                                      GTK_PLOT_LINE_SOLID,
                                      0, prf.color[num]);

 sprintf (name, "Function %i", num+1);

 gtk_plot_dataset_set_legend(dataset[num], name);

 gtk_plot_hide_dataset (dataset[num]); /* Hahah!! */
}


void set_error (char *s)
{
  char *h, *t;
  h = (char *) malloc (300);
  gtk_label_get (GTK_LABEL(errlab), &h);
  /* append */
  t = malloc (strlen (s) + strlen(h) + 1);
  sprintf (t, "%s%s", h, s);
  gtk_label_set_text (GTK_LABEL(errlab), t);
}


int main(int argc, char *argv[]){

 GtkWidget *vbox1, *hbox1, *vbox2, *vbox3;
 GtkWidget *scrollw1;
 GtkWidget *menubar;
 GtkWidget *button;
 GtkWidget *label;
 GtkWidget *menu, *menu_item;
 GtkWidget *scale;
 GtkWidget *sep1;
 GtkWidget *frame;
 GtkObject *adj;

 int i, j[10];

 double width;
 int w, height;

 extern char *errtext;

 /* Inits...... Imlib is used in exporting the plot. */
 gtk_init(&argc,&argv);
 gdk_imlib_init();

 /* Part of imlib initialization... no idea why */

 gtk_widget_push_visual(gdk_imlib_get_visual());
 gtk_widget_push_colormap(gdk_imlib_get_colormap());


 errtext = (char *) malloc (40);

 /* Initialize parser symbol table */
 init_table ();

 /* Set up evaluation window */
 calc_new ();


 /* Make a new window, connect its close box to a quit function */

 window1=gtk_window_new(GTK_WINDOW_TOPLEVEL);
 gtk_window_set_title(GTK_WINDOW(window1), OURNAME);
 gtk_widget_set_usize(window1,PREFS_DEFAULT_WINDOW_X, PREFS_DEFAULT_WINDOW_Y);
 gtk_container_border_width(GTK_CONTAINER(window1),0);

 gtk_signal_connect (GTK_OBJECT (window1), "destroy",
		     GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
 gtk_signal_connect (GTK_OBJECT (window1), "key-press-event",
		     GTK_SIGNAL_FUNC (callback_keypress), NULL);

 vbox1=gtk_vbox_new(FALSE,0);
 gtk_container_add(GTK_CONTAINER(window1),vbox1);
 gtk_widget_show(vbox1);

 /* Menubar init code. */
 get_main_menu (vbox1, &menubar);
 gtk_box_pack_start (GTK_BOX(vbox1), menubar, FALSE, TRUE, 0);
 gtk_widget_show (menubar);

 /* We have 2 hboxes, the main vbox for the window, and the box in the top for the entry, option menu, and button. */

 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);

 /* Create and initialize the option menu */

 menu = gtk_menu_new ();
 for (i=0; i!=10; ++i)
   {
     gchar label[12];
     strcpy (label, _("Function"));
     sprintf (label, "%s %i", label, i+1); /* We store i and more importantly formn as 0-9. The option meny should display 1-10. */
     menu_item = gtk_menu_item_new_with_label (label);
     gtk_menu_append (GTK_MENU (menu), menu_item);
     j[i] = i; /* STUPID hack... we want to pass the function number to the callback handler. But we need to pass a pointer to a variable. And we cant pass &i becuase i goes from 1 to 10 so it will be 10 when this actually matters. I'm too tired and lazy to think of a more intelligent soulution. */
     gtk_signal_connect (GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC (callback_popup), &j[i]);
     gtk_widget_show (menu_item);
     
     formc[i] = (char *) malloc (256);
     strcpy (formc[i], ""); /* it's a 0-9 loop anyway, perfect time to initalize the function strings with nulls. */
   }
 gtk_widget_show (menu);

 
 opmenu = gtk_option_menu_new ();
 gtk_option_menu_set_menu (GTK_OPTION_MENU(opmenu), menu);
 gtk_option_menu_set_history (GTK_OPTION_MENU(opmenu), 0);
 gtk_box_pack_start (GTK_BOX(hbox1), opmenu, FALSE, FALSE, 0);
 gtk_widget_show (opmenu);

 edit = gtk_entry_new_with_max_length(255);
 gtk_box_pack_start (GTK_BOX(hbox1), edit, FALSE, FALSE, 0);
 gtk_widget_show (edit);
 
 /* Set up entry and then Graph button */

 button = gtk_button_new_with_label (_("Graph"));
 gtk_box_pack_start (GTK_BOX(hbox1), button, FALSE, FALSE, 0);
 gtk_widget_show (button);
 gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (callback_button), NULL);


 button = gtk_button_new_with_label (_("Controls"));
 gtk_box_pack_end (GTK_BOX(hbox1), button, FALSE, FALSE, 0);
 gtk_widget_show (button);
 gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (callback_ctl_show), NULL);

 sep1 = gtk_hseparator_new ();
 gtk_box_pack_start (GTK_BOX(vbox1), sep1, FALSE, TRUE, 0);
 gtk_widget_show (sep1);

 /* Create scrolled window. TODO: Get rid of this without fucking up the gtkplotcanvas */

 scrollw1=gtk_scrolled_window_new(NULL, NULL);
 gtk_container_border_width(GTK_CONTAINER(scrollw1),0);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw1),
 				GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);
 gtk_box_pack_start(GTK_BOX(vbox1),scrollw1, TRUE, TRUE,0);
 gtk_widget_show(scrollw1);

 /* Make and set up the canvas that the gtkplot will draw into */

 canvas = gtk_plot_canvas_new(500, 500);
 GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(canvas), GTK_PLOT_CANVAS_DND_FLAGS);
 GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(canvas), GTK_PLOT_CANVAS_ALLOCATE_TITLES);
 GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(canvas), GTK_PLOT_CANVAS_CAN_RESIZE_PLOT);
 GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(canvas), GTK_PLOT_CANVAS_CAN_MOVE_PLOT);
 layout = canvas;
 
 
 /*frame = gtk_frame_new(NULL);
 gtk_container_set_border_width(GTK_CONTAINER(frame), 1);
 gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
 gtk_widget_show (frame);*/

 //gtk_container_add (GTK_CONTAINER(frame), layout);
 gtk_container_add(GTK_CONTAINER(scrollw1),layout);
 gtk_box_pack_start(GTK_BOX(vbox1),layout, TRUE, TRUE, 0);
 gtk_layout_set_size(GTK_LAYOUT(layout), 500, 500);
 GTK_LAYOUT(layout)->hadjustment->step_increment = 5;
 GTK_LAYOUT(layout)->vadjustment->step_increment = 5;

 /*button = gtk_button_new();
 gtk_widget_set_usize (button, 30, 60);
 gtk_layout_put (GTK_LAYOUT(canvas), button, 500 - */
 /* connect signal */
 

 /* DND setup */
 //gtk_drag_source_set (GTK_WIDGET(layout), GDK_BUTTON1_MASK|GDK_BUTTON3_MASK, target_table, 1, GDK_ACTION_COPY);
 //gtk_signal_connect (GTK_OBJECT (layout), "drag_data_get", GTK_SIGNAL_FUNC (callback_source_drag_data_get), NULL);


 ctlwin=gtk_window_new(GTK_WINDOW_DIALOG);
 gtk_container_border_width(GTK_CONTAINER(window1),0);


 hbox1=gtk_hbox_new (FALSE, 0);
 // gtk_box_pack_start (GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
 gtk_container_add (GTK_CONTAINER(ctlwin), hbox1);
 gtk_container_set_border_width (GTK_CONTAINER(hbox1), 5);
 gtk_widget_show (hbox1);

 vbox2=gtk_vbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(hbox1), vbox2, FALSE, FALSE, 0);
 gtk_widget_show (vbox2);

 vbox3=gtk_vbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(hbox1), vbox3, FALSE, FALSE, 0);
 gtk_container_set_border_width (GTK_CONTAINER(vbox3), 20);
 gtk_widget_show (vbox3);

 vbox3=gtk_vbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(hbox1), vbox3, FALSE, FALSE, 0);
 gtk_widget_show (vbox3);

 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry2 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry2), "0.5");
 gtk_widget_set_usize (entry2, 60, entry2->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry2, FALSE, FALSE, 0);
 gtk_widget_show (entry2);


 label = gtk_label_new (_("Scroll delta (X):   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox3), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry3 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry3), "0.5");
 gtk_widget_set_usize (entry3, 60, entry3->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry3, FALSE, FALSE, 0);
 gtk_widget_show (entry3);


 label = gtk_label_new (_("Scroll delta (Y):   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry6 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry6), "-1");
 gtk_widget_set_usize (entry6, 60, entry6->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry6, FALSE, FALSE, 0);
 gtk_widget_show (entry6);


 label = gtk_label_new (_("X minimum:   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox3), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry7 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry7), "1");
 gtk_widget_set_usize (entry7, 60, entry7->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry7, FALSE, FALSE, 0);
 gtk_widget_show (entry7);


 label = gtk_label_new (_("X maximum:   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry8 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry8), "-1");
 gtk_widget_set_usize (entry8, 60, entry8->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry8, FALSE, FALSE, 0);
 gtk_widget_show (entry8);


 label = gtk_label_new (_("Y minimum:   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox3), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry9 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry9), "1");
 gtk_widget_set_usize (entry9, 60, entry9->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry9, FALSE, FALSE, 0);
 gtk_widget_show (entry9);


 label = gtk_label_new (_("Y maximum:   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 entry4 = gtk_entry_new_with_max_length(8);
 gtk_entry_set_text (GTK_ENTRY(entry4), "2");
 gtk_widget_set_usize (entry4, 60, entry4->requisition.height);
 gtk_box_pack_end (GTK_BOX(hbox1), entry4, FALSE, FALSE, 0);
 gtk_widget_show (entry4);


 label = gtk_label_new (_("Zoom delta:   "));
 gtk_box_pack_end (GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 gtk_widget_show (label);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_box_pack_start (GTK_BOX(vbox3), hbox1, FALSE, FALSE, 0);
 gtk_widget_show (hbox1);


 button = gtk_button_new_with_label (_("Close"));
 gtk_box_pack_end (GTK_BOX(hbox1), button, FALSE, FALSE, 0);
 gtk_widget_show (button);
 gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (callback_ctl_hide), NULL);


 button = gtk_button_new_with_label (_("Apply"));
 gtk_box_pack_end (GTK_BOX(hbox1), button, FALSE, FALSE, 0);
 gtk_widget_show (button);
 gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (callback_range_apply), NULL);


 /*adj = gtk_adjustment_new (0, -5, 6, 1, 1, 1);
 gtk_signal_connect (GTK_OBJECT(adj), "value_changed", GTK_SIGNAL_FUNC(callback_zoom_slider), NULL);

 scale = gtk_hscale_new (GTK_ADJUSTMENT(adj));
 gtk_scale_set_digits (GTK_SCALE(scale), 0);
 gtk_box_pack_end (GTK_BOX(hbox1), scale, TRUE, TRUE, 0);
 gtk_widget_show (scale);*/


 sep1 = gtk_hseparator_new ();
 gtk_box_pack_start (GTK_BOX(vbox1), sep1, FALSE, TRUE, 0);
 gtk_widget_show (sep1);


 frame = gtk_frame_new(NULL);
 gtk_container_set_border_width(GTK_CONTAINER(frame), 1);
 gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
 gtk_widget_show (frame);


 hbox1=gtk_hbox_new (FALSE, 0);
 gtk_container_add (GTK_CONTAINER(frame), hbox1);
 gtk_widget_show (hbox1);


 errlab = gtk_label_new ("Ready.");
 gtk_widget_set_usize (errlab, hbox1->requisition.width-5, errlab->requisition.height);
 gtk_box_pack_start (GTK_BOX(hbox1), errlab, FALSE, TRUE, 0);
 gtk_widget_show (errlab);


 gtk_widget_show(layout);

 active_plot = new_layer(canvas);


 /* Set up the 10 functions */
 for (i = 0; i != 10; ++i)
 {
   build_dataset (i);
 }

 /* Initialize the defaults. Load in from a prefs file if prefs file if possible, otherwise use defaults from prefs.h */
 defaults();
 if (!prefs_read())
   prefs_save ();




 /* Set grid attributes. We do this in initialization of the program even if the user doesn't have grids enabled so they will be set up already if the user tuns them on. Make a dashed line for major and turn off minor.*/
 gtk_plot_set_major_vgrid_attributes (GTK_PLOT(active_plot), GTK_PLOT_LINE_DOTTED, 1, prf.gridcolor);
 gtk_plot_set_major_hgrid_attributes (GTK_PLOT(active_plot), GTK_PLOT_LINE_DOTTED, 1, prf.gridcolor);
 gtk_plot_set_minor_vgrid_attributes (GTK_PLOT(active_plot), GTK_PLOT_LINE_DOTTED, 1, prf.gridcolor);
 gtk_plot_set_minor_hgrid_attributes (GTK_PLOT(active_plot), GTK_PLOT_LINE_DOTTED, 1, prf.gridcolor);

 /* Set other stuff */

  gtk_plot_set_range(GTK_PLOT(active_plot), rangexmi, rangexma, rangeymi, rangeyma);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .08, .05);
  gtk_plot_hide_legends_border(GTK_PLOT(active_plot));
  gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
  gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT);
  gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, majorticks_h, minorticks_h);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, majorticks_v, minorticks_v);
  gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, majorticks_v, minorticks_v);


  /* This function handles all setting of a GtkPlot that are indicated by preferences.*/
  reset_flags();

 
 gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .125, .125);
 gtk_widget_show(active_plot);


 gtk_plot_axis_set_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, "X Title");
  gtk_plot_text_get_size (GTK_PLOT(active_plot)->bottom.title, &w, &height);
  width = w;
  gtk_plot_axis_move_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, 0, 0.5 - (width/1000), GTK_PLOT(active_plot)->bottom.title.y);
  

  gtk_plot_axis_set_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, "Y Title");
  gtk_plot_text_get_size (GTK_PLOT(active_plot)->left.title, &height, &w);
  width = w;
  gtk_plot_axis_move_title (GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, 0, GTK_PLOT(active_plot)->left.title.x, 0.5 - (width/1000));


 gtk_plot_layout_put_text(GTK_PLOT_LAYOUT(canvas), .45, .04, 0, 
                          "Helvetica", 12, NULL, NULL,
                          "Plot title");

 
 gtk_widget_show (window1);

 gtk_main();

 return(0);
}


/* This gets called when we reset prefs or initialize them. It makes the pref changes take effect on the current widgets. */
void reset_flags (void)
{
// gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15);
// gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 0, 1., .5);

 gtk_plot_set_background (GTK_PLOT(active_plot), prf.bgcolor);

 if (prf.showtop)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_TOP_AXIS);
   }
 else
   {
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_TOP_AXIS);
   }
 if (prf.showright)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_RIGHT_AXIS);
   }
 else
   {
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_RIGHT_AXIS);
   }
 if (prf.showleft)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_LEFT_AXIS);
   }
 else
   {
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_LEFT_AXIS);
   }
 if (prf.showbottom)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_BOTTOM_AXIS);
   }
 else
   {
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_BOTTOM_AXIS);
   }
 
 if (prf.showxy)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_X0);
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_Y0);
   }
 else
  {
    GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_X0);
    GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_Y0);
  }
 if (prf.showgrid)
   {
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_V_GRID);
     GTK_PLOT_SET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_H_GRID);
   }
 else
   {
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_V_GRID);
     GTK_PLOT_UNSET_FLAGS(GTK_PLOT(active_plot), GTK_PLOT_SHOW_H_GRID);
   }
 
 if (prf.showlegend) gtk_plot_show_legends (GTK_PLOT(active_plot));
 else gtk_plot_hide_legends (GTK_PLOT(active_plot));

 gtk_widget_queue_draw (active_plot);
}


