/* Copyright (C) 1999 Aaron Lehmann
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



#include "mathparse.tab.c"

#include <gdk/gdk.h>

struct mp_stack *entryidx[2][2][11];

short initial, flagend=0;
short currset=0;

extern gint8      formn;
extern char     *formc[10];

char *c;

extern void set_error (char *);


static struct mp_stack * mp_new_entry (int stacknum);
static void mp_free_entry (int stacknum);


int yyerror (char *s)  /* Dud */
{
  return (1);
}

int yyerror_real (char *s)  /* Called by yyparse on error */
{
  parseerr = 1;
  if (currset < 10)
    {
      sprintf (errtext, "Function %i: %s.  ", currset+1, s);
      set_error (errtext);
    }

  return (1);
}

struct init
{
  char *fname;
  double (*fnct)();
};

struct initc
{
  char *cname;
  double cvalue;
};


struct init arith_fncts[]
  =
{
  { "sin",    sin    },
  { "asin",   asin   },
  { "cos",    cos    },
  { "acos",   acos   },
  { "tan",    tan    },
  { "atan",   atan   },
  { "ln",     log    },
  { "log",    log10  },
  { "exp",    exp    },
  { "sqrt",   sqrt   },
  { "abs",    fabs   },
  { "tanh",   tanh   },
  { "atanh",  atanh  },
  { "sinh",   sinh   },
  { "asinh",  asinh  },
  { "cosh",   cosh   },
  { "acosh",  acosh  },
  { "lgamma", lgamma },
  { "erf",    erf    },
  { "erfc",   erfc   },
  { 0,     0         }
};


struct initc constants[]
  = {
       { "e",  M_E   },
       { "pi", M_PI  },
       { 0,    0     }
    };


/* The symbol table: a chain of `struct symrec'.  */
symrec *sym_table = (symrec *)0;

void init_table ()  /* puts arithmetic functions in table. */
{
  int i;
  symrec *ptr;
  for (i = 0; arith_fncts[i].fname != 0; i++)
    {
      ptr = putsym (arith_fncts[i].fname, FNCT);
      ptr->value.fnctptr = arith_fncts[i].fnct;
    }

  for (i = 0; constants[i].cname != 0; i++)
    {
      ptr = putsym (constants[i].cname, CONST);
      ptr->value.var = constants[i].cvalue;
    }


  for (i=0; i != 11; ++i)
    {
      entryidx[0][0][i]=0;
      entryidx[0][1][i]=0;
      entryidx[1][0][i]=0;
      entryidx[1][1][i]=0;
    }
      
}


symrec *
putsym (sym_name,sym_type)
     char *sym_name;
     int sym_type;
{
  symrec *ptr;
  ptr = (symrec *) malloc (sizeof (symrec));
  ptr->name = (char *) malloc (strlen (sym_name) + 1);
  strcpy (ptr->name,sym_name);
  ptr->type = sym_type;
  ptr->value.var = 0; /* set value to 0 even if fctn.  */
  ptr->next = (struct symrec *)sym_table;
  sym_table = ptr;
  return ptr;
}


symrec *
getsym (sym_name)
     char *sym_name;
{
  symrec *ptr;
  for (ptr = sym_table; ptr != (symrec *) 0;
       ptr = (symrec *)ptr->next)
    if (strcmp (ptr->name,sym_name) == 0)
      return ptr;
  return 0;
}


int yylex ()
{
  if (flagend == 1) { flagend = 0; return (0); }


  if (!initial){ ++c; }

  initial = 0;


  if (*c == ' ' || *c == '\t')
    {
      /* Ignore whitespace, get first nonwhite character.  */
      while (*++c == ' ' || *c == '\t')
	;
    }

  if (*c == '\0')
    {
      flagend = 1;
      return ('\n');
    }
  
	/* Char starts a number => parse the number.         */
  if (*c == '.' || isdigit (*c))
    {
      sscanf (c, "%lf", &yylval.val);
      while (*c == '.' || isdigit (*c)) ++c;
      --c;
      return NUM;
    }

  /* Char starts an identifier => read the name.       */
  if (isalpha (*c))
    {
      symrec *s;
      static char *symbuf = 0;
      static int length = 0;
      int i;

      /* Initially make the buffer long enough
         for a 40-character symbol name.  */
      if (length == 0)
        length = 40, symbuf = (char *)malloc (length + 1);

      i = 0;
      do
        {
          /* If buffer is full, make it bigger.        */
          if (i == length)
            {
              length *= 2;
              symbuf = (char *)realloc (symbuf, length + 1);
            }
          /* Add this character to the buffer.         */
          symbuf[i++] = tolower(*c);
          /* Get another character.                    */
          ++c;
        }
      while (*c != '\0' && isalnum (*c));

      --c;

      symbuf[i] = '\0';

      s = getsym (symbuf);
      if (s == 0)
	{
	  /* Uh oh... it isn't a function - lets hope its a variable */
	  if (strlen (symbuf) == 1)
	    { /* We're OK */
	      s = putsym (symbuf, VAR);
	    }
	  else
	    { /* We're fucked */
	      /* The user entered a string which is longer than one char but not a function */
	      //ERROR
	      char *namesym;
	      namesym = (char *) malloc (strlen(symbuf) + 20);
	      sprintf (namesym, "Unknown identifier: %s", symbuf);
	      yyerror_real (namesym);
	      flagend = 1;
	      return ('\n');
	    }
	}	
      yylval.tptr = s;
      return s->type;
    }

  
  
  /* Any other character is a token by itself.        */
  return (*c);
}

void mp_push_op (short mptoken)
{
  struct mp_stack *p = mp_new_entry(0);
  p->tokentype = mptoken;
}


void mp_push_var (char mpvar)
{
  struct mp_stack *p = mp_new_entry(0);
  p->tokentype = MP_TOKEN_VARIABLE;
  p->id = mpvar;
}


void mp_push_num (double mpnum)
{
  struct mp_stack *p = mp_new_entry(0);
  p->tokentype = MP_TOKEN_CONSTANT;
  p->value = mpnum;
}


void mp_push_fnct (symrec *in)
{
  struct mp_stack *p = mp_new_entry(0);
  p->tokentype = MP_TOKEN_FNCT;
  p->symboltab = in;
}


void mp_push_const (symrec *in)
{
  struct mp_stack *p = mp_new_entry(0);
  p->tokentype = MP_TOKEN_CONST;
  p->symboltab = in;
}


void mp_set_eq (char mpname)
{
}


double mp_eval_exp (double x, int *err)
{
  struct mp_stack *p, *q;

  for (p=entryidx[0][0][currset]; p != 0; p=p->next)
    {
      switch (p -> tokentype)
	{
	case MP_TOKEN_VARIABLE:
	  if (p->id == 'x' && !err)
	    {
	      q = mp_new_entry(1);
	      q -> value = x;
	    }
	  else
	    {
	      if (err)
		{
		  *err = MP_ERROR_VARIABLE;
		}
	      return (0);
	    }
	  break;
	case MP_TOKEN_CONSTANT:
	  q = mp_new_entry(1);
	  q->value = p->value;
	  break;
	case MP_TOKEN_CONST:
	  q = mp_new_entry(1);
	  q->value = p->symboltab->value.var;
	  break;
	case MP_TOKEN_PLUS:
	  entryidx[1][1][currset]->previous->value = entryidx[1][1][currset]->previous->value + entryidx[1][1][currset]->value;
	  mp_free_entry (1);
	  break;
	case MP_TOKEN_MINUS:
	  entryidx[1][1][currset]->previous->value = entryidx[1][1][currset]->previous->value - entryidx[1][1][currset]->value;
	  mp_free_entry (1);
	  break;
	case MP_TOKEN_MULT:
	  entryidx[1][1][currset]->previous->value = entryidx[1][1][currset]->previous->value * entryidx[1][1][currset]->value;
	  mp_free_entry (1);
	  break;
	case MP_TOKEN_DIV:
	  entryidx[1][1][currset]->previous->value = entryidx[1][1][currset]->previous->value / entryidx[1][1][currset]->value;
	  mp_free_entry (1);
	  break;
	case MP_TOKEN_FACT:
	  entryidx[1][1][currset]->value = mp_fact (entryidx[1][1][currset]->value);
	  break;
	case MP_TOKEN_POW:
	  entryidx[1][1][currset]->previous->value = pow (entryidx[1][1][currset]->previous->value, entryidx[1][1][currset]->value);
	  mp_free_entry (1);
	  break;
	case MP_TOKEN_FNCT:
	  entryidx[1][1][currset]->value = (*(p->symboltab->value.fnctptr))(entryidx[1][1][currset]->value);
	  break;
	case MP_TOKEN_UNARY_MINUS:
	  /* Take the last number on the stack and reverse the sign */
	  entryidx[1][1][currset]->value *= (-1);
	  break;
	}
    }
  return entryidx[1][1][currset]->value;
}


void mp_fix_stack (void)
{
  struct mp_stack *p;
  for (p=entryidx[1][0][currset]; p != 0; p=p->next)
    {
      mp_free_entry (1);
    }
}
 

void mp_stack_clean (void)
{
  struct mp_stack *p;
  for (p=entryidx[0][0][currset]; p != 0; p=p->next)
    {
      mp_free_entry (0);
    }
}


double mp_fact (double in)
{
  int i;
  double out=in;
  in = ceil (in); /* Round to integer */
  for (i = 2; i < in; ++i) out *= i;
  return (out);
}


static struct mp_stack * mp_new_entry (int stacknum)
{
  struct mp_stack *p;
  p = (struct mp_stack *) malloc (sizeof (struct mp_stack));

  if (!p)
    {
      //ERROR
    }
  if (!entryidx[stacknum][1][currset])
    { /* No last entry. This entry will be the first in the list. */
      entryidx[stacknum][0][currset] = p;
      entryidx[stacknum][1][currset] = p;
      p->next = 0;
      p->previous = 0;
    }
  else
    {
      p->next = 0;
      p->previous = entryidx[stacknum][1][currset];
      entryidx[stacknum][1][currset]->next = p;
      entryidx[stacknum][1][currset] = p;
    }

  return (p);
}


static void mp_free_entry (int stacknum)
{
  struct mp_stack *p = entryidx[stacknum][1][currset];
  if (entryidx[stacknum][0][currset] == p)
    {
      entryidx[stacknum][0][currset] = 0;
      entryidx[stacknum][1][currset] = 0;
      free (p);
      return;
    }
  entryidx[stacknum][1][currset] = p->previous;
  free (p);
  entryidx[stacknum][1][currset]->next = 0;
}
